OpenTok Android SDK
===================

The OpenTok Android SDK lets you use OpenTok-powered video sessions in apps you build
for Android devices.

All OpenTok applications are composed of two parts:

* The client-side, which uses the [OpenTok client SDKs](https://tokbox.com/developer/sdks/client/)
  and runs in a user's browser or mobile app
* The server-side, which uses the [OpenTok server SDKs](https://tokbox.com/developer/sdks/server/)
  and runs on your server to pass authentication information to the client.

The client SDK for building Android apps is the OpenTok Android SDK, which provides most of the
core functionality for your app, including:

* Connecting to an OpenTok session
* Publishing streams to a session
* Subscribing to streams in a session

Client SDKs are also available for [iOS](https://tokbox.com/developer/sdks/ios/) and
[web](https://tokbox.com/developer/sdks/js/). All OpenTok client SDKs can interact with one another.

You can learn more about the basics of OpenTok clients, servers, sessions, and more on the
[OpenTok Basics](https://tokbox.com/developer/guides/basics/) page.

Learning to build with the OpenTok Android SDK
----------------------------------------------

The best way to learn how to use the OpenTok Android SDK is to follow the OpenTok [Basic Video Chat
tutorial](https://tokbox.com/developer/tutorials/android/).

Once you understand the basics of building with the OpenTok Android SDK, you can get more
detailed information and learn how to customize your application with the OpenTok [developer
guides](https://tokbox.com/developer/guides). To investigate specific API classes and methods,
see the [OpenTok Android SDK API reference](https://tokbox.com/developer/sdks/android/reference/).

Code samples
------------

For sample code, visit our
[opentok-android-sdk-samples repo](https://github.com/opentok/opentok-android-sdk-samples)
on GitHub.

Interoperability
----------------

Apps written with the OpenTok Android SDK 2.19.0 can interoperate with OpenTok apps
written with version 2.17+ of the OpenTok client SDKs:

* OpenTok.js
* OpenTok Android SDK
* OpenTok iOS SDK
* OpenTok Windows SDK
* OpenTok Linux SDK

Installation
------------

The library (an aar file) is included in the root of the SDK bundle.

A Maven version is available at https://bintray.com/tokbox/maven/opentok-android-sdk.
The artifact ID is `"opentok-android-sdk"`. For more information, see [Creating your own app
using the OpenTok Android SDK](#creating-your-own-app-using-the-opentok-android-sdk).

Developer and client requirements
---------------------------------

The OpenTok Android SDK supports one published audio-video stream, one subscribed audio-video
stream, and up to five additional subscribed audio-only streams simultaneously (this is the
baseline support on a Samsung Galaxy S3). To connect more than two clients in a session using the
OpenTok Android SDK, create a session that uses the OpenTok Media Router (a session with the
media mode set to routed). See [The OpenTok Media Router and media
modes](http://tokbox.com/developer/guides/create-session/#media-mode).

The SDK is supported on high-speed Wi-Fi and 4G LTE networks.

The OpenTok Android SDK is supported on armeabi-v7a, armeabi64-v8a, x86, and x86_64 architectures.

The OpenTok Android SDK works with any Android 4.1+ device (Jelly Bean, API Level 16) that has
a camera (for publishing video) and adequate CPU and memory support.

Creating your own app using the OpenTok Android SDK
---------------------------------------------------

The library is provided in the opentok-android-sdk-2.19.0.aar file. You no longer need to include
architecture-dependent .so files (which was required in previous versions). This file is included in
the root of the SDK package. Place the aar file into your project in Android Studio. Then modify the
gradle file for your project to reference the aar file.

A Maven version is available at https://bintray.com/tokbox/maven/opentok-android-sdk.
The artifact ID is `"opentok-android-sdk"`. Modify your app to download the OpenTok Android SDK
from http://tokbox.bintray.com/maven. For example:

1. Edit the build.gradle for your project and add the following code snippet to the
   `allprojects/repositories` section:

  `maven { url  "http://tokbox.bintray.com/maven" }`

2. Modify the app's build.gradle file and add the following code snippet to the
   `dependencies` section:

  `implementation 'com.opentok.android:opentok-android-sdk:2.19.0'`

3. Modify the app's build.gradle file and add the following code snippet to the
   `android` section:

  ```
  compileOptions {
      sourceCompatibility JavaVersion.VERSION_1_8
      targetCompatibility JavaVersion.VERSION_1_8
  }
  ```

Your app needs to use a session ID and token generated with your OpenTok API key, which you can get
by logging into your [TokBox account](https://tokbox.com/account).

For test purposes, you can generate a session ID and token by logging into your
[TokBox account](https://tokbox.com/account). For a production app, generate unique tokens
(and session IDs, if you need to support multiple sessions) using the
[OpenTok server-side libraries](https://tokbox.com/developer/sdks/server).

Permissions
-----------

The OpenTok Android SDK uses following permissions:

* `android.permission.CAMERA` -- If your app does not use the default video capturer
   and does not access the camera, you can remove this permission.

* `android.permission.INTERNET` -- Required.

* `android.permission.RECORD_AUDIO` -- If your app does not use the default audio
   device and does not access the microphone, you can remove this permission.

* `android.permission.MODIFY_AUDIO_SETTINGS` -- If your app does not use the default audio
   device and does not access the microphone, you can remove this permission.

* `android.permission.BLUETOOTH` -- The the default audio device supports
   Bluetooth audio. If your app does not use the default audio device and does not
   use Bluetooth, you can remove this permission.

* `android.permission.BROADCAST_STICKY` -- We have determined that this is unused by the
  OpenTok Android SDK, and we will remove this permission from an upcoming release.

* `android.permission.READ_PHONE_STATE` -- The OpenTok Android SDK requests this permission
  in API level 22 and lower.

You do not need to add these to your app manifest. The OpenTok Android SDK adds them automatically.
However, in API level 21 and higher, certain permissions require you to prompt the user.

Your app can remove any of these permissions that will not be required.
See [this post](https://stackoverflow.com/a/31616472) and [this
Android documentation](https://developer.android.com/studio/build/manifest-merge).
For example, this removes the `android.permission.CAMERA` permission:

```
<uses-permission android:name="android.permission.CAMERA" tools:node="remove"/>
```

Documentation
-------------

Reference documentation is available in the docs subdirectory of the SDK and at
<http://www.tokbox.com/developer/sdks/android/reference/index.html>.

More information
-----------------

For a list of new features and known issues, see the [release notes](release-notes.md).
