OpenTok Android SDK release notes
=================================

New features and changes
------------------------

Version 2.19.1 -- April 2021

* This version fixes an issue where a potential memory leak may occur when publishing stops.

Version 2.19.0 -- February 2021

* This version of the SDK uses WebRTC 84, which includes improvements in performance and stability.

* This version adds support for RTC audio and video statistics for publishers and subscribers.
  See the docs for the  `OTPublisherKit.getRtcStatsReport()` and `OTSubscriberKit.getRtcStatsReport()`
  methods.

* This version fixes the following issues:

  * The audio bitrate value set for publishers was handled incorrectly internally.

  * The SDK was not able to publish in Pixel 4 devices running API Level 30.

Version 2.18.1 -- October 2020

* This version fixes the following issues:

  * The `PublisherKit.onError()` callback was not being called in some scenarios where publishing failed.

  * The `BaseVideoCapturer.provideBufferFrame()` method was not interpreting some formats correctly.

  * The SDK was not proxying all communications correctly when the IP Proxy feature was being used.

  * The SDK ProGuard rules were causing conflicts with other libraries.
  
  * The SDK ProGuard rules were marking internal attributes as public.

Version 2.18.0 -- August 2020

* This version adds support for the [IP Proxy](https://tokbox.com/developer/guides/ip-proxy/) feature.

* This version adds the following methods and properties:

  - `PublisherKit.onStop()`
  - `PublisherKit.onRestart()`
  - `BaseVideoCapturer.onStop()`
  - `BaseVideoCapturer.onRestart()`

  Please see the documentation for details.

* From this version on, RGBA video frames are supported in custom video capturers.

* This version fixes the following issues:

    * Apps could crash when disconnecting from a session when calling the `PublisherKit.finalize()` method.

    * A regression in the Frame Metadata API where the metadata was not being transmitted could cause apps to crash when using the metadata associated with a video frame.

    * Restore the `Stream.compareTo()` method, which was removed in 2.17.0.

    * The Android video view was not properly rendered when changing from scale type FILL to FIT.

    * An error getting the type of video for publishers.

Version 2.17.2 -- July 2020

* This version fixes an issue that could cause apps to crash while trying to initiate audio capturing.

Version 2.17.1 -- June 2020

* This version fixes the following issues:

  * The `sessionId`  property was null for a Session instance returned
    from calling `Publisher.getSession()` or `Subscriber.getSession()`.

  * `AudioDeviceManager.getAudioDevice().getOutputMode()` returned the wrong value
    when a Bluetooth device was connected prior to an app connecting to
    an OpenTok session.

Version 2.17.0 -- April 2020

* This version adds the following methods and properties:

  - `PublisherKit.onAudioLevelUpdated(float)`
  - `SubscriberKit.onAudioEnabled()`
  - `SubscriberKit.onAudioDisabled()`
  - `SubscriberKit.StreamListener.onAudioDisabled()`
  - `SubscriberKit.StreamListener.onAudioEnabled()`
  - `BaseVideoCapturer.CaptureSettings.mirrorInLocalRender`
  - `BaseAudioDevice.getBluetoothState()`

  Please see the documentation for details.

* This version deprecates the following APIs:

  - `Session.Builder.setApiUrl(URL)`
  - `PublisherKit.destroy()`
  - `SubscriberKit.destroy()`
  - `SubscriberKit.onDisconnected()`
  - `Stream.getSession()`
  - `Publisher.startPreview()`

* This version can be used along with Google's WebRTC Android library in the same application.

Version 2.16.6 -- March 2020

* This version fixes the following issues:

  * Apps could crash while trying to publish without a valid camera interface.

  * Apps could crash due to a bug in the default audio device implementation.

  * Apps could experience issues capturing and rendering audio when the device receives an incoming phone call.

  * The following WebRTC properties are now omitted for logging when the IP Proxy feature is used: `audioLocalAddress`, `audioRemoteAddress`, `videoLocalAddress`, and `videoRemoteAddress`.

Version 2.16.5 -- February 2020

* This version fixes the following issues:

  * Apps could crash when pausing and resuming audio.

  * We removed some URLs from the code that were preventing apps from receiving
    FirstNet certification.

  * Calling `Session.onResume()` before calling `Session.onPause()` could result
    in video dropping out.

Version 2.16.4 -- February 2020

* This version fixes the following issues:

  * Apps could crash when a disconnecting from a session if a PSTN call was in progress.

  * The SDK was not proxying all HTTP requests when the IP Proxy feature was being used.

  * The SDK was reporting incorrect values for the publisher's stream properties after the stream properties had changed.

Version 2.16.3 -- December 2019

* This version fixes the following issues:

  * Apps could crash when disconnecting from a session with more than one subscriber.

  * Apps could crash because some callbacks threw a NullPointerException.

Version 2.16.2 -- September 2019

* This version fixes the following issues:

  * Data for published streams is not showing up in
    [OpenTok Inspector](https://tokbox.com/developer/tools/inspector/).

  * Apps crash when calling the `SubscriberKit.getStream()` method in
    the `SubscriberKit.onDisconnect()` callback, because the method throws
    a NullPointerException.

  * Apps crash or freeze when destroying publishers and subscribers after
    disconnecting from a session.

  * When using TextureView, if a publisher or subscriber view is removed
    from the parent layout, adding the view again results in black views
    for both publishers and subscribers.

  * Unsubscribing a subscriber when its stream is dropped results in an error.

  * When using a TextureView, publisher and subscriber views were not being
    displayed when an app came into the foreground from the background
    on devices running Android 6 or below.

  * Apps could crash shortly after connecting to an OpenTok session.

  * When communicating with the OpenTok logging infrastructure, the SDK was creating a temporary
   `cert.pem` file.

Version 2.16.1 -- June 2019

* This version fixes the following issues:

  * The `hasAudio` and `hasVideo` properties of a subscriber's `stream` property
    were not updated when the stream's publisher unpublished audio and video.

  * The SDK could freeze when disconnecting from a session after disconnecting
    a publisher.

  * Video rendering was not resizing correctly when a TextureView containing the
    video was resized.

  * There were issues with the frame rate and autofocus when using the back camera.

Version 2.16.0 -- April 2019

* When using this version, you will need to specify `sourceCompatibility` and
  `targetCompatibility` to use Java 8. You can do this by modifying the app's
  build.gradle file and adding the following code snippet to the `android` section:

  ```
  compileOptions {
      sourceCompatibility JavaVersion.VERSION_1_8
      targetCompatibility JavaVersion.VERSION_1_8
  }
  ```

* The `android.hardware.camera.autofocus` feature is no longer required in the
  Android manifest for your app.

* This version of the SDK uses WebRTC 71, which includes improvements
  in performance and stability.

* This version fixes the following issues:

  * Using a device's volume button while an app using the OpenTok SDK is in
    the background resulted in the incorrect UI being displayed on the device.
    This also could cause other issues, such as errors if tries to record video.

  * If you connected to a session with a subscriber token and attempted
    to publish, the application would crash.

  * `Subscriber.getStream()` would return `null` when the
    `SessionListener.onStreamDropped()` method is called.

Version 2.15.3 -- January 2019

* The OpenTok Android SDK now requests the
  `android.permission.READ_PHONE_STATE` permission. This additional
  permission was added in version 2.15.0. Your app can remove
  any permissions requested by the OpenTok Android SDK that it
  does not use. See [this post](https://stackoverflow.com/a/31616472) and [this
  [Android documentation](https://developer.android.com/studio/build/manifest-merge).

* This version fixes the following issues:

  * In Android 4.x, an application could crash if it tried
    to remove the `android.permission.READ_PHONE_STATE`
    permission that is now used by the SDK.

  * Calling `Session.getCapabilities()` would cause an application to crash.

Version 2.15.2 -- December 2018

* This version fixes the following issues:

  * Handset audio output (using `BaseAudioDevice.OutputMode.Handset`)
    and other audio output issues.

  * Reconnecting to sessions would occasionally fail.

Version 2.15.1 -- November 2018

* This version fixes an issue in which a subscriber would not receive audio
  or video from a stream that was initially published without audio or video
  and subsequently had them enabled (by the publisher).

Version 2.15.0 -- October 2018

* The new `MediaUtils.getSupportedCodecs(Context)` method
  lets you determine the video codecs supported on the device.

* The `SubscriberKit.onVideoDisabled()` method can be called with the `reason`
  parameter set to the new `SubscriberKit.VIDEO_REASON_CODEC_NOT_SUPPORTED`
  value, which indicates that a subscriber's video was disabled because the
  stream uses a video codec (such as H.264) not supported on the device.

* The new `PublisherKit.setAudioStatsListener()` and
  `PublisherKit.setVideoStatsListener()` methods let you monitor audio
  and video statistics for a publisher.

* This version adds support for the x86_64 architecture. This improves performance
  on devices like the Vuzix M300.

* [Enterprise partners](https://www.vonage.com/communications-apis/video/pricing/) that have IP white listing
  enabled for an OpenTok project should call the new `Session.Builder.ipWhitelist()`
  method (passing in `true`) when instantiating a Session object.

Version 2.14.2 -- August 2018

* This version fixes a critical security vulnerability in the HTTP library. It is recommended that customers update to incorporate this fix.

Version 2.14.1 -- July 2018

* This version fixes the following issues:

  * When resizing a TextureView containing an OpenTok video, the resized view does not
    scale the whole video correctly and results in a cropped video. However, when resizing
    a GLSurfaceView containing an OpenTok video, the resized view produces a properly scaled
    video. This issue was introduced in v2.14.0.

  * Sessions sometimes fail to [automatically
    reconnect](https://tokbox.com/developer/guides/connect-session/android/#automatic_reconnection)
    This issue was introduced in v2.14.0.

Version 2.14.0 -- April 2018

* This version adds a `metadata` parameter to each of the BaseVideoCapturer methods for
  providing video frames to a custom video capturer: `BaseVideoCapturer.provideByteArrayFrame()`,
  `BaseVideoCapturer.provideIntArrayFrame()`, `BaseVideoCapturer.provideBufferFrame()`, and
  `BaseVideoCapturer.provideBufferFramePlanar()`. When implementing a custom video capturer,
  you can set this parameter to provide metadata for a video frame. (For backwards compatibility,
  the BaseVideoCapturer class retains the existing versions of each of these methods, which
  do not include the `metadata` parameter.) When implementing a custom video renderer, you can
  use the new `Frame.getMetadata()` method to get the metadata.

* This version fixes the following issues:

  * Calling `Subscriber.getSession()` method after the Session object is destroyed could cause
    an app to crash.

  * The `videoPacketsLost` value reported by the `SubscriberKit.VideoStatsListener.onVideoStats()`
    method was always set to 0.

  * The SDK failed to work on networks with proxy servers.

Version 2.13.0 -- January 2018

* This version adds H.264 support on devices that support H.264 encoding (either via
  hardware or software). Software encoding is only supported on newer Android versions
  running Android 6+ (Marshmallow). This means that Android clients can interoperate
  with Safari clients (and others) in [Safari OpenTok projects](https://tokbox.com/developer/sdks/js/safari/).

* This version fixes the following issues:

  * Subscribing to high-resolution video would sometime result in cropping of the video.

  * Videos that use the scalable video feature would sometimes magnify.
    Scalable video is supported in sessions that use the [OpenTok media
    router](https://tokbox.com/developer/guides/create-session/#media-mode).

Version 2.12.1 -- October 2017

* This version includes improvements for performance and stability.

Version 2.12.0 -- September 2017

* Android 8.0 (Oreo) support.

* This version fixes the following issues:

  * Publishing an audio-only stream after previously publishing video fails.

  * An error getting camera parameters when using the default camera capturer can
    cause the app to crash.

  * Audio output might go to the phone speaker when it is set to use the headset speaker.

  * Reusing a publisher without unpublishing from the first session can cause
    the app to crash.

Version 2.11.1 -- June 2017

* This version corrects the frame rate used by some cameras. In previous versions, some cameras
  used a frame rate that caused the video to be dark.

Version 2.11.0 -- April 2017

* Audio-tuning API. Use the `PublisherKit.Builder.audioBitrate(bitsPerSecond)` method to set the
  audio bitrate for a publisher. (For more information, see the reference documentation.)

* This version adds errors for stream and connection limits. These errors are useful in large
  OpenTok sessions:

  * If you call `Session.connect()` when the session's connection limit has been exceeded,
    the `Session.SessionListener.onError()` method is called with an OTError object
    that has an error code set to `OpentokError.ErrorCode.SessionConnectionLimitExceeded`.

  * If you call `Session.Subscribe()` when the session's stream limit has been exceeded,
    the `SubscriberKit.SubscriberListener.onError()` method is called with an OTError object
    that has an error code set to `OpentokError.ErrorCode.SubscriberStreamLimitExceeded`.

  See the [beta programs](https://tokbox.com/developer/beta/) page for
  information on supporting sessions with up to 3,000 real-time viewers.

* The `Session.SessionOptions.isHwDecodingSupported()` method is deprecated, and the method
  has no functionality. The SDK no longer supports using the Android MediaCodec class to
  decode video.

* This version of the SDK uses WebRTC 56, which includes improvements in performance and stability.

This version fixes the following issues:

* Cameras used by OpenTok publishers are now released when you call `Session.onPause()`.
  Publishers reacquire cameras when you call `Session.onResume()`.

* The camera display of a publisher was too dark on the Google Pixel. We added further improvements
  to those made in the 2.10.1 release.

Version 2.10.1

This version fixes the following issues:

* The camera display of a publisher was too dark on the Google Pixel.

* Switching the audio driver to use handset (`BaseAudioDevice.OutputMode.Handset`) was not working.

* The `PublisherKit.setCapturer(capturer)` method did not work properly.

Version 2.10.0

* This version adds new builder classes for constructing Session, Publisher,
  PublisherKit, Subscriber, and SubscriberKit objects:

  * `Session.Builder`
  * `Publisher.Builder`
  * `PublisherKit.Builder`
  * `Subscriber.Builder`
  * `SubscriberKit.Builder`

  These classes let you define any combination of initialization settings to be used when creating
  Session, Publisher, PublisherKit, Subscriber, and SubscriberKit objects.

  The existing constructor methods of the Session, Publisher, PublisherKit, Subscriber, and
  SubscriberKit classes are deprecated.

* This version of the OpenTok Android SDK is supported on Android 4.1+ (Jelly Bean, API Level 16).
  For a list of supported devices, see the [Developer and client
  requirements](http://tokbox.com/developer/sdks/android/#developer-and-client-requirements)
  section of the OpenTok Android SDK page at the TokBox developer center.

* This version fixes an issue in which the video for a publisher was mirrored when using the back
  camera (on some devices).

Version 2.9.1

* This version fixes an issue with automatic reconnection when switching between networks.
  For more information on automatic reconnection, see
  https://tokbox.com/developer/guides/connect-session/android/#automatic_reconnection.

Version 2.9.0

* This version adds 64-bit support (the armeabi64-v8a architecture).

* The OpenTok Media Router now includes the OpenTok scalable video feature
  (previously in beta). For more information, see the
  [OpenTok Media Router](https://tokbox.com/developer/guides/create-session/#media-mode)
  documentation.

* This version adds a `Session.getCapabilities()` method. Use this method
  to check whether the client can publish streams to an OpenTok session,
  based on the role assigned to the token used to connect to the session.

* Please upgrade your apps to use the latest versions of the [OpenTok server
  SDKs](https://tokbox.com/developer/sdks/server/). These now use JSON web
  tokens (JWT) for authentication. JWT provides increased security along with
  standards-based encoding, decoding, and verification. The old form of
  authentication will expire in July 2017. Also, if you use the OpenTok REST API
  (instead of the server SDKs), please use JWT to authenticate calls to the
  OpenTok REST methods (see [this topic](https://tokbox.com/developer/rest/#authentication)
  on authentication).

Version 2.8.2

* Fixed bug where the publisher audio-video state was not preserved between application
  lifecycle events.

* Fixed a potential memory leak in subscribers.

* Fixed a Bluetooth routing issue on certain devices.

* Fixed bug in which custom video renderers resulted in black videos.

Version 2.8.1

* This version fixes an audio-video synchronization issue in version 2.8.0.

* Note that we have added TLS support to OpenTok TURN servers. This fixes
  media streaming for clients behind network firewalls that block non-TLS
  traffic over port 443.

Version 2.8.0

* The library is now provided as an aar file (not a jar file).

* This version add support for Bluetooth SCO profile devices (earpieces and headsets) in the default
  audio driver.

* This version improves quality for published audio.

* The new `Publisher.startPreview()` method lets you display a publisher's camera video in the
  Publisher's view before it starts streaming video.

* The new `Session.SessionOptionsProvider` interface lets you decide when to use hardware video
  decoding.

* The `Publisher.swapCamera()`, `Publisher.getCameraId()`, and `Publisher.setCameraId()` methods
  are deprecated. You should use the new `Publisher.cycleCamera()` method to cycle between cameras,
  if there are multiple cameras on the device. Use the new `BaseVideoCapturer.CaptureSwitch`
  interface to define the behavior of the `Publisher.cycleCamera()` method in a custom video
  capturer.

* The `PublisherKit.setCapturer(BaseVideoCapturer capturer)` method is deprecated. Instead, use the
  `PublisherKit.capturer` property when creating a custom PublisherKit implementation. Or use the
  new `Publisher(Context context, String name, BaseVideoCapturer capturer)` constructor.

* This version of the SDK uses WebRTC 49.

Version 2.7.0

* Setting the frame rate and resolution for a published stream using the default camera video
  capturer. A new version of the Publisher constructor, `Publisher(context, name, resolution, fps)`,
  includes parameters for setting the frame rate and resolution of the stream.

* New methods for getting audio and video statistics for a stream. You can now set listeners to
  monitor the following statistics for a subscriber's stream:

  * Total audio and video packets lost
  * Total audio and video packets received
  * Total audio and video bytes received

  See the documentation for the `SubscriberKit.setAudioStatsListener(AudioStatsListener listener)`
  and `SubscriberKit.setVideoStatsListener(VideoStatsListener listener)` methods.

* Added support for armeabi-v7a.

* This version requires Android 4.1 (Jelly Bean, API Level 16) or later.

Version 2.6.0

* Added the `Session.reportIssue()` method. You can call this method when your app
  experiences an issue. The method returns an issue ID, which you can use with the
  <a href="https://tokbox.com/developer/tools/Inspector">Inspector</a> or when
  discussing an issue with the TokBox support team.

* This release fixes the following issues:

  * Subscribers did not display video when the stream changed from audio-only to audio-video.

  * Calling the `Session.sendSignal()` method when not connected to the session resulted in
    the app crashing.

  * Calling the `getData()` method of the Connection object passed into the
    `Session.SignalListener.onSignalReceived(session, type, data, connection)` method did
    not return connection data.

  * When network connections were re-routed through a proxy or when Session objects were
    garbage collected, fatal signal 11 (SIGSEGV) errors would occur..

  * Reference table overflow errors caused apps to crash.

Version 2.5.0

* Added proxy support.

* Note that the samples directory is no longer included in the SDK bundle. The sample code is now
  available at the open-source
  [opentok-android-sdk-samples repository](https://github.com/opentok/opentok-android-sdk-samples)
  on GitHub. This allows us to keep it up to date and provide developers with latest version of
  the sample code. Feel free to clone the repo or download the source code to see the best-practice
  examples of OpenTok usage.

* Added hardware-accelerated VP8 decoding of videos on the Nexus 5 with Android Lollipop+.

* Fixed a bug that could occur when you reuse a Session object to reconnect to a session.

* Added a `SessionIllegalState` error in the `OpentokError.ErrorCode` enum. This defines an error
  code for an error that occurs if you try to connect to a session that is already connected or if
  you try to subscribe to a stream that is no longer in the session.

Version 2.4.1

* Fixed an issue with failing DTLS negotiations with newer versions of Firefox,
  which can cause publishing and subscribing to fail in relayed OpenTok sessions
  (sessions that do not use [the OpenTok Media
  Router](http://tokbox.com/opentok/tutorials/create-session/#media-mode ) ).

Version 2.4.0

* This version adds support for testing on the Genymotion x86 emulator and the official Android x86
  emulator images in combination with the Intel HAXM software for VM Acceleration.

* This version adds support for screen sharing. When publishing a screen-sharing stream,
  call the `setPublisherVideoType(type)` method of the PublisherKit object and pass in
  `PublisherKitVideoType.PublisherKitVideoTypeScreen`. This optimizes the video encoding
  for screen sharing. It is recommended to use a low frame rate (5 frames per second or lower)
  with screen sharing. 	When using the
  screen video type in a session that uses the OpenTok Media Server, you should
  call the `setAudioFallbackEnabled(boolean enabled)` method of the Publisher object and
  pass in <code>true</code> to disable the audio-only fallback feature, so that the
  video does not drop out in subscribers. See [the OpenTok Media Router and media
  modes](http://tokbox.com/opentok/tutorials/create-session/#media-mode).

  When a stream is created in a session, you can determine the video type of the stream
  (screen or camera) by calling the `getStreamVideoType()` method of the Stream object.
  The type of stream is defined by constants in the StreamVideoType enum: `StreamVideoTypeScreen`
  and `StreamVideoTypeCamera`.

  To publish a screen-sharing stream, you need to implement a custom video capturer for the
  OTPublisherKit object. For sample code that publishes a screen-sharing stream, see the
  "screensharing" sample (in the samples directory).

* You can disable the audio-only fallback feature for a published stream by calling the
  `setAudioFallbackEnabled(enabled)` method of the PublisherKit object (passing in `false`).
  The audio-fallback feature is available in sessions that use the [OpenTok Media
  Router](http://tokbox.com/opentok/tutorials/create-session/#media-mode). With the audio-fallback
  feature enabled (the default), when the OpenTok Media Router determines that a stream's quality
  has degraded significantly for a specific subscriber, it disables the video for that subscriber
  in order to preserve audio quality. The audio-fallback feature is enabled by default.

* This version of the SDK takes advantage of hardware acceleration for video decoding on devices
  that support it. These devices include the Nexus 5 (with OS before 5.0 Lollipop), the Asus K010,
  the Asus K011, and the Asus K013.

* Updated to use BoringSSL, Google's fork of OpenSSL.

* Fixed an issue with video not recovering when network conditions improve (after video was
  disabled, due to network conditions, in a session using the OpenTok Media Server).

Version 2.3.0

* The reason parameter has been added to the
  `SubscriberKit.VideoListener.onVideoDisabled(SubscriberKit subscriber, String reason)`
  and `SubscriberKit.onVideoDisabled(SubscriberKit subscriber, String reason)`
  methods. This parameter describes the reason why the subscriber video is being
  disabled. In the previous version, this method was only called when the video
  was disabled due to changes in the stream quality (in a session that uses the
  OpenTok Media Router). In version 2.3.0, the method is also called if the
  publisher stops sending a video stream or the subscriber stops subscribing to
  it (and the `reason` parameter value will be set accordingly).

* New methods indicate when a subscriber's video stream starts (when there
  previously was no video) or resumes (after video was disabled):
  `SubscriberKit.VideoListener.onVideoEnabled(SubscriberKit subscriber, String reason)`
  and `SubscriberKit.onVideoEnabled(SubscriberKit subscriber, String reason)`.

* Use the new PublisherKit.AudioLevelListener and SubscriberKit.AudioLevelListener
  classes to monitor changes in the audio levels of publishers and subscribers.
  To use these, call the `setAudioLevelListener(listener)` method of a PublisherKit
  or SubscriberKit object.

* The new `SubscriberKit.VideoListener.onVideoDisabledWarning(subscriber)` method
  is called when the OpenTok Media Router determines that the stream quality has
  degraded and the video will be disabled if the quality degrades more. The new
  `SubscriberKit.VideoListener.onVideoDisabledWarningLifted(subscriber)` method
  is called when the stream quality improves. This feature is only available in
  sessions that use the OpenTok Media Router (sessions with the
  [media mode](http://tokbox.com/opentok/tutorials/create-session/#media-mode)
  set to routed), not in sessions with the media mode set to relayed. The
  SubscriberKit class also has protected `onVideoDisabledWarning()` and
  `onVideoDisabledWarningLifted()` methods that you can implement (instead of
  the VideoListener methods) when you subclass SubscriberKit.

* This version includes a new custom audio driver API. This lets you use
  custom audio streams and define the audio device used to capture and render
  audio data. The following new classes support the custom audio driver API:

  * AudioDeviceManager -- Use this class to set the app to specify a custom
  audio device for use in the app.

  * BaseAudioDevice -- Defines an audio device for use in a session.

  * BaseAudioDevice.AudioBus -- The audio bus marshals audio data between the network and
  the audio device.

  * BaseAudioDevice.AudioSettings -- Defines the format of the audio.

* You can optimize the speaker usage for voice-only calls by calling
  `AudioDeviceManager.getAudioDevice().setOutputMode(BaseAudioDevice.VOICE_COMMUNICATION)`.
  This sets the app to use the headset speaker (and the loudspeaker is disabled), which is
  preferable in voice-only apps.

* This build improves the video quality under poor network conditions.

* The Session.PublisherListener and the `Session.onPublisherAdded(publisher)`
  and `Session.onPublisherRemoved(publisher)` methods have been removed. You
  can monitor the `PublisherKit.PublisherListener.onStreamCreated(stream)` and
  `PublisherKit.PublisherListener.onStreamDestroyed(stream)` methods to determine
  when a publisher's stream is created and destroyed.


Version 2.2.1

* Updated to use version 1.0.1h of OpenSSL.

* The `Session.unpublish()` method now takes a PublisherKit parameter, instead of a Publisher
  parameter. (The Publisher class extends PublisherKit.)

Known issues
------------

* In a session with the [media mode](http://tokbox.com/opentok/tutorials/create-session/#media-mode)
  set to relayed, only one client can subscribe to a stream published by an Android device.

* Video streaming is prevented on networks that have firewalls that use authenticated
  proxies. This is due to a core issue with the current underlying WebRTC implementation.
  (See [this Chromium bug report](https://code.google.com/p/chromium/issues/detail?id=439560).)

* Audio is distorted in streams published from the Genymotion emulator.
